@@ -1,8 +1,10 @@
 #include "evopediaapplication.h"
 
 #include <QTranslator>
+#include <QLibraryInfo>
 
 #include "mainwindow.h"
+#include "utils.h"
 
 EvopediaApplication::EvopediaApplication(int &argc, char **argv) :
     QApplication(argc, argv)
@@ -12,9 +14,14 @@ EvopediaApplication::EvopediaApplication(int &argc, char **argv) :
 #endif
 
     QTranslator *qtTranslator = new QTranslator(this);
+    qtTranslator->load("qt_" + QLocale::system().name(), QLibraryInfo::location(QLibraryInfo::TranslationsPath));
+    installTranslator(qtTranslator);
+
+    qtTranslator = new QTranslator(this);
     qtTranslator->load(":tr/evopedia_" + QLocale::system().name());
     installTranslator(qtTranslator);
 
+
     m_evopedia = new Evopedia(this);
     m_mainwindow = new MainWindow();
 
@@ -32,6 +39,9 @@ EvopediaApplication::~EvopediaApplication()
 
 int main(int argc, char *argv[])
 {
+    /* initialize random number generator */
+    randomNumber(2);
+
     EvopediaApplication app(argc, argv);
     return app.exec();
 }
