@@ -3,6 +3,7 @@
 
 #include <QPointF>
 #include <QDataStream>
+#include <QPair>
 
 #include "title.h"
 
@@ -13,6 +14,12 @@ public:
     GeoTitle(Title title, QPointF coordinate) : title(title), coordinate(coordinate) {}
     const Title &getTitle() const { return title; }
     const QPointF &getCoordinate() const { return coordinate; }
+
+    static bool nearerThan(const QPair<GeoTitle, float> &t1, const QPair<GeoTitle, float> &t2)
+    {
+        return t1.second < t2.second;
+    }
+
 private:
     Title title;
     QPointF coordinate;
