@@ -3,6 +3,12 @@
 
 #include <QMainWindow>
 
+#if defined(USE_MOBILITY)
+#include <QGeoPositionInfoSource>
+#include <QGeoPositionInfo>
+QTM_USE_NAMESPACE
+#endif
+
 #include "map.h"
 
 namespace Ui {
@@ -16,10 +22,13 @@ public:
     explicit MapWindow(QWidget *parent = 0);
     ~MapWindow();
 
-    /* TODO1 request current GPS position */
     void setPosition(qreal lat, qreal lng, int zoom=-1);
     void getPosition(qreal &lat, qreal &lng, int &zoom);
 
+public slots:
+#if defined(USE_MOBILITY)
+    void positionUpdated(const QGeoPositionInfo &posInfo);
+#endif
 private slots:
     void delayedInit() {
 #if defined(Q_OS_SYMBIAN)
@@ -35,7 +44,16 @@ private:
     void grabZoomKeys(bool grab);
 #endif
 
+#if defined(USE_MOBILITY)
+    QGeoPositionInfoSource *posSource;
+#endif
+
     Ui::MapWindow *ui;
+
+private slots:
+    void on_actionFollow_GPS_toggled(bool );
+    void on_actionUse_GPS_toggled(bool );
+    void on_actionGo_to_GPS_Position_triggered();
 };
 
 #endif // MAPWINDOW_H
