@@ -2,7 +2,7 @@
 
 #include <QByteArray>
 
-#include "storagebackend.h"
+#include "localarchive.h"
 
 TitleIterator::TitleIterator()
     : device(0), prefix(QString())
@@ -11,7 +11,7 @@ TitleIterator::TitleIterator()
 }
 
 TitleIterator::TitleIterator(QIODevice *device_ini, const QString &prefix, const QString &language)
-    : language(language), device(device_ini), prefix(StorageBackend::normalize(prefix))
+    : language(language), device(device_ini), prefix(LocalArchive::normalize(prefix))
 {
     checkHasNext();
 }
@@ -29,7 +29,7 @@ void TitleIterator::checkHasNext()
         QByteArray line = device->readLine();
         nextTitle = Title(line.left(line.length() - 1), language);
         if (!prefix.isNull()) {
-            QString tn = StorageBackend::normalize(nextTitle.getName());
+            QString tn = LocalArchive::normalize(nextTitle.getName());
             if (!tn.startsWith(prefix))
                 nextTitle = Title();
         }
